<?php
//  Copyright (c) 2015 Daniel Robert Bradley. All rights reserved.
//  This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

class JStream
{
    static function Encode( $indent, $something )
    {
        //echo str_repeat( " ", $indent * 4 );

        if ( is_array( $something ) )
        {
            self::EncodeArray( $indent, $something );
        }
        else
        {
            self::EncodeObject( $indent, $something );
        }
    }

    static function EncodeResults( $indent, $results )
    {
        echo "{";
        
        if ( is_array( $results ) && (0 < count( $results ) ) )
        {
            echo '"results":';
            echo "[";
            
            $sep = "";
            
            foreach ( $results as $tuple )
            {
                echo $sep;
                self::EncodeTuple( $index + 1, $tuple );
                $sep = ",";
            }
            
            echo "]";
        }
        
        echo "}";
    }

    static function EncodeTuple( $indent, $tuple )
    {
        echo "\n" . str_repeat( " ", $indent * 4 ) . "{";
        $sep = "";
    
        foreach ( $tuple as $key => $value )
        {
            echo $sep;

            self::EncodeStringValue( $indent + 1, $key, $value );

            $sep = ",";
        }

        echo "\n" . str_repeat( " ", $indent * 4 ) . "}";
    }

    static function EncodeArray( $indent, $array )
    {
        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "[";
        $sep = "";
        
        if ( self::is_assoc( $array ) )
        {
            foreach ( $array as $string => $value )
            {
                echo $sep . "{";
                
                self::EncodeStringValue( $indent + 1, $string, $value );
                
                echo "}";

                $sep = ",";
            }
        }
        else
        {
            foreach ( $array as $value )
            {
                echo $sep;
                
                self::EncodeValue( $indent + 1, $value );
                
                $sep = ",";
            }
        }

        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "]";
    }

    static function EncodeObject( $indent, $object )
    {
        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "{";
        $sep = "";

        foreach ( $object as $member => $value )
        {
            echo $sep;
            
            self::EncodeStringValue( $indent + 1, $member, $value );

            $sep = ",";
        }
        echo "\n" . str_repeat( " ", $indent * 4 );
        echo "}";
    }

    static function EncodeStringValue( $indent, $string, $value )
    {
        echo "\n" . str_repeat( " ", $indent * 4 );
        self::EncodeString( $indent, $string );
        echo ":";
        self::EncodeValue( $indent, $value );
    }

    static function EncodeString( $indent, $string )
    {
        $escaped = json_encode( $string ); // str_replace( "\\", "\\\\", $string );

        if ( false === $escaped )
        {
            error_log( "ERROR1: " . var_export( $string,  true ) );
            $escaped = json_encode( htmlentities( $string ) );
        }

        if ( false === $escaped )
        {
            error_log( "ERROR2: " . var_export( $string,  true ) );
            $escaped = json_encode( "" );
        }

        echo $escaped;
    }

    static function EncodeValue( $indent, $value )
    {
        if ( true === $value )
        {
            echo "true";
        }
        else
        if ( false === $value )
        {
            echo "false";
        }
        else
        if ( is_array( $value ) )
        {
            self::EncodeArray( $indent, $value );
        }
        else
        if ( is_string( $value ) )
        {
            self::EncodeString( $indent, $value );
        }
        else
        if ( is_numeric( $value ) )
        {
            self::EncodeNumber( $indent, $value );
        }
        else
        if ( is_null( $value ) )
        {
            echo "null";
        }
        else
        {
            self::EncodeObject( $indent, $value );
        }
    }

    static function EncodeNumber( $indent, $number )
    {
        echo json_encode( $number );
    }

    static function is_assoc( $a )
    {
        $assoc = true;
    
        $keys = array_keys( $a );
        foreach ( $keys as $key )
        {
            if ( is_numeric( $key ) && (0 == $key) ) $assoc = false;
            break;
        }
        return $assoc;
    }

    static function is_assocx( $a )
    {
        $b = array_keys($a);

        return ($a != array_keys($b));
    }
}

